@@ -1,7 +1,7 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 #
 # Copyright (c) 2002 - 2003 Apple Inc.
-# Copyright (c) 2004 - 2013 The MacPorts Project
+# Copyright (c) 2004 - 2016, 2018 The MacPorts Project
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,7 @@ set org.macports.archivefetch [target_new org.macports.archivefetch portarchivef
 #target_init ${org.macports.archivefetch} portarchivefetch::archivefetch_init
 target_provides ${org.macports.archivefetch} archivefetch
 target_requires ${org.macports.archivefetch} main
+target_runtype ${org.macports.archivefetch} always
 target_prerun ${org.macports.archivefetch} portarchivefetch::archivefetch_start
 
 namespace eval portarchivefetch {
@@ -59,12 +60,12 @@ default archivefetch.ignore_sslcert no
 default archivefetch.pubkeys {$archivefetch_pubkeys}
 
 default archive_sites {[portarchivefetch::filter_sites]}
-default archive_sites.listfile {"archive_sites.tcl"}
-default archive_sites.listpath {"port1.0/fetch"}
+default archive_sites.listfile archive_sites.tcl
+default archive_sites.listpath port1.0/fetch
 default archive.subdir {${subport}}
 
 proc portarchivefetch::filter_sites {} {
-    global prefix frameworks_dir applications_dir porturl \
+    global prefix_frozen frameworks_dir_frozen applications_dir_frozen porturl \
         cxx_stdlib delete_la_files \
         portfetch::mirror_sites::sites portfetch::mirror_sites::archive_type \
         portfetch::mirror_sites::archive_prefix \
@@ -83,7 +84,7 @@ proc portarchivefetch::filter_sites {} {
         set $key $val
     }
 
-    set ret {}
+    set ret [list]
     foreach site [array names portfetch::mirror_sites::archive_prefix] {
         set missing 0
         foreach var {archive_frameworks_dir archive_applications_dir archive_type archive_cxx_stdlib archive_delete_la_files} {
@@ -96,9 +97,9 @@ proc portarchivefetch::filter_sites {} {
             continue
         }
         if {$portfetch::mirror_sites::sites($site) ne {} &&
-            $portfetch::mirror_sites::archive_prefix($site) eq $prefix &&
-            $portfetch::mirror_sites::archive_frameworks_dir($site) eq $frameworks_dir &&
-            $portfetch::mirror_sites::archive_applications_dir($site) eq $applications_dir &&
+            $portfetch::mirror_sites::archive_prefix($site) eq $prefix_frozen &&
+            $portfetch::mirror_sites::archive_frameworks_dir($site) eq $frameworks_dir_frozen &&
+            $portfetch::mirror_sites::archive_applications_dir($site) eq $applications_dir_frozen &&
             $portfetch::mirror_sites::archive_cxx_stdlib($site) eq $cxx_stdlib &&
             $portfetch::mirror_sites::archive_delete_la_files($site) eq $delete_la_files &&
             ![catch {archiveTypeIsSupported $portfetch::mirror_sites::archive_type($site)}]} {
@@ -108,7 +109,7 @@ proc portarchivefetch::filter_sites {} {
     }
 
     # check if porturl itself points to an archive
-    if {[file rootname [file tail $porturl]] eq [file rootname [get_portimage_name]] && [file extension $porturl] ne ""} {
+    if {![catch {get_portimage_name} portimage_name] && [file rootname [file tail $porturl]] eq [file rootname $portimage_name] && [file extension $porturl] ne ""} {
         lappend ret [string range $porturl 0 end-[string length [file tail $porturl]]]:[string range [file extension $porturl] 1 end]
         archive.subdir
     }
@@ -150,7 +151,7 @@ proc portarchivefetch::get_full_archive_sites_path {} {
 proc portarchivefetch::checkfiles {urls} {
     upvar $urls fetch_urls
 
-    portfetch::checksites [list archive_sites [list {} {} ARCHIVE_SITE_LOCAL]] \
+    portfetch::checksites [list archive_sites [list {} ARCHIVE_SITE_LOCAL]] \
                           [get_full_archive_sites_path]
     checkarchivefiles fetch_urls
 }
@@ -181,7 +182,7 @@ proc portarchivefetch::fetchfiles {args} {
         dropPrivileges
     }
 
-    set fetch_options {}
+    set fetch_options [list]
     if {[string length ${archivefetch.user}] || [string length ${archivefetch.password}]} {
         lappend fetch_options -u
         lappend fetch_options "${archivefetch.user}:${archivefetch.password}"
@@ -195,7 +196,7 @@ proc portarchivefetch::fetchfiles {args} {
     if {$portverbose eq "yes"} {
         lappend fetch_options "--progress"
         lappend fetch_options "builtin"
-    } elseif {[llength [info commands ui_progress_download]] > 0} {
+    } else {
         lappend fetch_options "--progress"
         lappend fetch_options "ui_progress_download"
     }
@@ -236,15 +237,15 @@ proc portarchivefetch::fetchfiles {args} {
                     curl fetch --effective-url effectiveURL {*}$fetch_options $file_url "${incoming_path}/${archive}.TMP"
                     set fetched 1
                     break
-                } catch {{POSIX SIG SIGINT} eCode eMessage} {
+                } trap {POSIX SIG SIGINT} {_ eOptions} {
                     ui_debug [msgcat::mc "Aborted fetching archive due to SIGINT"]
                     file delete -force "${incoming_path}/${archive}.TMP"
-                    throw
-                } catch {{POSIX SIG SIGTERM} eCode eMessage} {
+                    throw [dict get $eOptions -errorcode] [dict get $eOptions -errorinfo]
+                } trap {POSIX SIG SIGTERM} {_ eOptions} {
                     ui_debug [msgcat::mc "Aborted fetching archive due to SIGTERM"]
                     file delete -force "${incoming_path}/${archive}.TMP"
-                    throw
-                } catch {{*} eCode eMessage} {
+                    throw [dict get $eOptions -errorcode] [dict get $eOptions -errorinfo]
+                } on error {eMessage} {
                     ui_debug [msgcat::mc "Fetching archive failed: %s" $eMessage]
                     set lastError $eMessage
                     file delete -force "${incoming_path}/${archive}.TMP"
@@ -295,7 +296,7 @@ proc portarchivefetch::fetchfiles {args} {
     if {[info exists archive_exists]} {
         # modify state file to skip remaining phases up to destroot
         global target_state_fd
-        foreach target {fetch checksum extract patch configure build destroot} {
+        foreach target {archivefetch fetch checksum extract patch configure build destroot} {
             write_statefile target "org.macports.${target}" $target_state_fd
         }
         return 0
