@@ -44,8 +44,6 @@
 #include "entry.h"
 #include "entryobj.h"
 #include "file.h"
-#include "graph.h"
-#include "item.h"
 #include "portgroup.h"
 #include "registry.h"
 #include "util.h"
@@ -87,29 +85,42 @@ int registry_tcl_detach(Tcl_Interp* interp, reg_registry* reg,
         reg_error* errPtr) {
     reg_entry** entries;
     reg_file** files;
-    int entry_count;
-    int file_count;
+    reg_portgroup** portgroups;
+    int count;
     int i;
-    entry_count = reg_all_open_entries(reg, &entries);
-    if (entry_count == -1) {
+
+    count = reg_all_open_entries(reg, &entries);
+    if (count == -1) {
         return 0;
     }
-    for (i=0; i<entry_count; i++) {
+    for (i = 0; i < count; i++) {
         if (entries[i]->proc) {
             Tcl_DeleteCommand(interp, entries[i]->proc);
         }
     }
     free(entries);
-    file_count = reg_all_open_files(reg, &files);
-    if (file_count == -1) {
+
+    count = reg_all_open_files(reg, &files);
+    if (count == -1) {
         return 0;
     }
-    for (i = 0; i < file_count; i++) {
+    for (i = 0; i < count; i++) {
         if (files[i]->proc) {
             Tcl_DeleteCommand(interp, files[i]->proc);
         }
     }
     free(files);
+
+    count = reg_all_open_portgroups(reg, &portgroups);
+    if (count == -1) {
+        return 0;
+    }
+    for (i = 0; i < count; i++) {
+        if (portgroups[i]->proc) {
+            Tcl_DeleteCommand(interp, portgroups[i]->proc);
+        }
+    }
+    free(portgroups);
     if (!reg_detach(reg, errPtr)) {
         return registry_failed(interp, errPtr);
     }
@@ -204,6 +215,7 @@ static int registry_open(ClientData clientData UNUSED, Tcl_Interp* interp,
         if (reg == NULL) {
             return TCL_ERROR;
         } else if (reg_attach(reg, path, &error)) {
+            reg_configure(reg);
             return TCL_OK;
         } else {
             return registry_failed(interp, &error);
@@ -226,6 +238,10 @@ static int registry_close(ClientData clientData UNUSED, Tcl_Interp* interp,
                 reg_vacuum(Tcl_GetAssocData(interp, "registry::db_path", NULL));
                 Tcl_DeleteAssocData(interp, "registry::needs_vacuum");
             }
+            /* Not really anything we can do if this fails. */
+            if (reg_checkpoint(reg, &error) == 0) {
+                fprintf(stderr, "%s\n", error.description);
+            }
             if (registry_tcl_detach(interp, reg, &error)) {
                 return TCL_OK;
             }
@@ -386,8 +402,6 @@ int Registry_Init(Tcl_Interp* interp) {
     Tcl_CreateObjCommand(interp, "registry::close", registry_close, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::read", registry_read, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::write", registry_write, NULL, NULL);
-    /* Tcl_CreateObjCommand(interp, "registry::graph", GraphCmd, NULL, NULL); */
-    /* Tcl_CreateObjCommand(interp, "registry::item", item_cmd, NULL, NULL); */
     Tcl_CreateObjCommand(interp, "registry::entry", entry_cmd, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::file", file_cmd, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::portgroup", portgroup_cmd, NULL, NULL);
